#############################################################################
#
#	Pyro Naming Service Control Tool
#
#	This is part of "Pyro" - Python Remote Objects
#	which is (c)1999 Irmen de Jong - irmen@bigfoot.com.
#
#############################################################################

from naming import NameServerLocator, NamingError
import Pyro.core
import sys



class PyroNSControl:
	def args(self, args):
		import Pyro.util
		self.Args = Pyro.util.ArgVParser()
		self.Args.parse(args)

	def connect(self, sysCmd=None):
		try:
			host = self.Args.getArg('-h')
		except KeyError:
			host = None
		try:
			port = int(self.Args.getArg('-p'))
		except KeyError:
			# select the default broadcast port
			port = Pyro.config.PYRO_NS_BC_PORT
		if host:
			print 'Trying NS @',host,port
		else:
			print 'Finding NS using broadcast @ port',port

		locator = NameServerLocator()
		if not sysCmd:
			self.NS = locator.getNS(host,port,1)
			print 'NS is at',self.NS.URI.host,'port',self.NS.URI.port
		else:
			result = locator.sendSysCommand(sysCmd,host,port,1)
			print 'NS replied to system command',sysCmd,':',result
		
	def ping(self):
		self.connect()
		self.NS.ping()
		print 'NS is up and running!'

	def list(self):
		self.connect()
		dict=self.NS.status()
		print '-------------- START DATABASE'
		keys=dict.keys()
		keys.sort()
		for i in keys:
			print i,' --> ',dict[i]
		print '-------------- END'

	def resolve(self):
		self.connect()
		for n in self.Args.argsR:
			print n,' -->',
			try:
				print self.NS.resolve(n)
			except NamingError:
				print '## not found ##'
	
	def register(self):
		self.connect()
		uri=Pyro.core.PyroURI(self.Args.argsR[1])
		try:
			self.NS.register(self.Args.argsR[0],uri)
			print 'registered',self.Args.argsR[0],' --> ',uri
		except NamingError,arg:
			print 'Error from NS:',arg
		
	def remove(self):
		self.connect()
		for n in self.Args.argsR:
			self.NS.unregister(n)
			print n,'unregistered.'

	def shutdown(self):
		self.connect(sysCmd='shutdown')


def usage():
	print 'PyroNS control program - usage is as follows;'
	print '>> nsc command [-h host] [-p port] [args...]'
	print 'where command is one of: ping, list, resolve, register, remove, shutdown'
	print '      host is the host where the NS should be contacted'
	print '      port is the non-standard Pyro NS port'
	print '      args... depend on the command.'
	raise SystemExit

def main():
	if __name__!='__main__':
		del sys.argv[0]	# remove '-c' arg

	if len(sys.argv)<1:
		usage()
	
	cmd = sys.argv[0]
	
	ctrl = PyroNSControl()
	ctrl.args(sys.argv[1:])

	try:
		# nice construct to map commands to the member function to call
		{ 'ping': ctrl.ping,
		  'list': ctrl.list,
		  'resolve': ctrl.resolve,
		  'register': ctrl.register,
		  'remove': ctrl.remove,
		  'shutdown': ctrl.shutdown } [cmd] ()
	except KeyError:
		usage()

